package blufi.ble;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.content.Context;

import blufi.ble.cb.BleReadCallback;
import blufi.ble.cb.OtherException;
import blufi.ble.params.BlufiConfigureParams;
import blufi.ble.params.ServiceUuid;
import blufi.ble.response.BleVersionResponse;

import blufi.ble.cb.*;

public class BleClient {

    public static final String VERSION = BuildConfig.VERSION_NAME;

    private static BleClient sIntance;
    private BleClientImpl mImpl;

    public BleClient(Context context, BluetoothDevice device, String uuidPreference, ServiceUuid serviceUuid) {
        mImpl = new BleClientImpl(this, context, device,uuidPreference,serviceUuid);
        sIntance = this;
    }

    public static BleClient getInstance() {
        return sIntance;
    }

    /**
     * Set BluetoothGattCallback
     *
     * @param callback the BluetoothGattCallback
     */
    public void setGattCallback(BluetoothGattCallback callback) {
        mImpl.setGattCallback(callback);
    }

    /**
     * Set the callback
     *
     * @param callback the BleCallback
     */
    public void setBleCallback(BleCallback callback) {
        mImpl.setBleCallback(callback);
    }

    /**
     * Set the business_callback
     *
     * @param callback the BleCallback
     */
    public void setBusinessCallback(BusinessCallback callback) {
        mImpl.setBusinessCallback(callback);
    }

    /**
     * Establish a BLE connection with BluetoothDevice
     */
    public void connect() {
        mImpl.connect();
    }

    /**
     * Close the client
     */
    public void close() {
        mImpl.close();
    }

    /**
     * Request to get device version. The result will notified in
     * {@link BleCallback#onDeviceVersionResponse(BleClient, int, BleVersionResponse)}
     */
    public void requestDeviceVersion() {
        mImpl.requestDeviceVersion();
    }

    /**
     * Request to get device current status. The result will be notified in
     * {@link BleCallback#(BleClient, int, )}
     */
    public void requestDeviceStatus() {
        mImpl.requestDeviceStatus();
    }

    /**
     * Negotiate security with device. The result will be notified in
     * {@link BleCallback#onNegotiateSecurityResult(BleClient, int)}
     */
    public void negotiateSecurity() {
        mImpl.negotiateSecurity();
    }

    /**
     * Configure the device to a station or soft AP. The posted result will be notified in
     * {@link BleCallback#onConfigureResult(BleClient, int)}
     *
     * @param params the config parameter
     */
    public void configure(final BlufiConfigureParams params) {
        mImpl.configure(params);
    }

    public void setMtu(final int mtu) {
        mImpl.setMtu(mtu);
    }

    public int getMtu() {
        return mImpl.getMtu();
    }

    /**
     * Request to get wifi list that the device scanned. The wifi list will be notified in
     */
    public void requestDeviceWifiScan() {
        mImpl.requestDeviceWifiScan();
    }

    /**
     * Request to post custom data to device. The posted result will be notified in
     * {@link BleCallback#onPostCustomDataResult(BleClient, int, String)}
     *
     * @param data the custom data
     */
    public void postCustomData(byte[] data) {
        mImpl.postCustomData(data);
    }

    public void postOtaBinInfo(byte[] data,int type) {
        mImpl.postOtaBinInfo(data,type);
    }

    public void postOtaBinData(byte[] data) {
        mImpl.postOtaBinData(data);
    }

    /**
     * Request device to disconnect the ble connection
     */
    public void requestCloseConnection() {
        mImpl.requestCloseConnection();
    }

    public BluetoothGatt getGatt() {
        return mImpl.getGatt();
    }

    public void read(BluetoothDevice bleBluetooth,
                     String uuid_service,
                     String uuid_read,
                     BleReadCallback callback) {
        if (callback == null) {
            throw new IllegalArgumentException("BleReadCallback can not be Null!");
        }

        if (bleBluetooth == null) {
            callback.onReadFailure(new OtherException("This device is not connected!"));
        } else {
            mImpl.readCharacteristic(uuid_service, uuid_read ,callback);
        }
    }

    public void write(BluetoothDevice bleDevice, String uuid_service,
                      String uuid_write, byte[] data, BleWriteCallback bleWriteCallback) {
        if (bleWriteCallback == null) {
            throw new IllegalArgumentException("BleWriteCallback can not be Null!");
        }

        if (data == null) {
            bleWriteCallback.onWriteFailure(new OtherException("data is Null!"));
            return;
        }
        //Use BluetoothManager#getConnectionState instead.
        //getGatt().getConnectionState(bleDevice) != BluetoothProfile.STATE_CONNECTED
        if (bleDevice == null) {
            bleWriteCallback.onWriteFailure(new OtherException("This device not connect!"));
        } else {
            if (data.length > getMtu() ) {
                bleWriteCallback.onWriteFailure(
                        new OtherException("Be careful: data's length beyond MTU so use spilt write!"));
            }else {
                mImpl.writeCharacteristic(uuid_service, uuid_write , data , bleWriteCallback);

            }
        }
    }

    public void notify(BluetoothDevice bleDevice,
                       String uuid_service,
                       String uuid_notify,
                       boolean useCharacteristicDescriptor,
                       BleNotifyCallback callback) {

    }

    public void destroyDetailCallback() {
        mImpl.destroyDetailCallback();
    }

    public void notify(BluetoothDevice bleDevice, String uuid_service, String uuid_notify, BleNotifyCallback callback) {
        if (callback == null) {
            throw new IllegalArgumentException("BleNotifyCallback can not be Null!");
        }
        mImpl.enableCharacteristicNotify(callback, uuid_service, uuid_notify, true);

    }

    /**
     * stop notify, remove callback
     *
     * @param uuid_service
     * @param uuid_notify
     * @return
     */
    public boolean stopNotify(BluetoothDevice bleBluetooth,
                              String uuid_service,
                              String uuid_notify) {
        if (bleBluetooth == null) {
            return false;
        }
        return mImpl.disenableCharacteristicNotify(uuid_service, uuid_notify, false);
    }

    /**
     * indicate
     *
     * @param bleDevice
     * @param uuid_service
     * @param uuid_indicate
     * @param callback
     */
    public void indicate(BluetoothDevice bleDevice,
                         String uuid_service,
                         String uuid_indicate,
                         BleIndicateCallback callback) {
        if (callback == null) {
            throw new IllegalArgumentException("BleIndicateCallback can not be Null!");
        }

        if (bleDevice == null) {
            callback.onIndicateFailure(new OtherException("This device not connect!"));
        } else {
            mImpl.enableCharacteristicIndicate(callback, uuid_service,uuid_indicate);
        }
    }

    /**
     * stop indicate, remove callback
     *
     * @param bleDevice
     * @param uuid_service
     * @param uuid_indicate
     * @return
     */
    public boolean stopIndicate(BluetoothDevice bleDevice,
                                String uuid_service,
                                String uuid_indicate) {
        if (bleDevice == null) {
            return false;
        }
        return mImpl.disableCharacteristicIndicate(uuid_service, uuid_indicate, false);
    }

    public void stopTask() {
        mImpl.stopTask();
    }

    public void send_ble_DATA_ZB_LINKKEY() {
        mImpl.send_ble_DATA_ZB_LINKKEY();

    }

    public void send_ble_CMD_ZB_RESET() {
        mImpl.send_ble_CMD_ZB_RESET();
    }
}
