package blufi.ble;

import android.annotation.TargetApi;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.widget.Toast;

import blufi.ble.cb.BleNotifyCallback;
import blufi.ble.cb.BleReadCallback;
import blufi.ble.cb.BleWriteCallback;
import blufi.ble.cb.*;
import blufi.ble.cb.OtherException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import blufi.ble.params.BlufiConfigureParams;
import blufi.ble.params.BlufiParameter;
import blufi.ble.params.ServiceUuid;
import blufi.ble.response.BleVersionResponse;

import static android.bluetooth.BluetoothDevice.BOND_BONDED;
import static blufi.ble.utils.ByteUtil.*;
import static blufi.ble.params.BlbleParameter.Type.ACKCode.SUBTYPE_ACK_SUCCESS;
import static blufi.ble.params.BlbleParameter.Type.ACKCode.SUBTYPE_ERROE_TYPE;
import static blufi.ble.params.BlbleParameter.Type.ACKCode.SUBTYPE_NOT_READY;

class BleClientImpl implements BlufiParameter,BleFuntions {
    private static final String TAG = "BleClientImpl";
    private static final boolean DEBUG = false;

    private BleClient mClient;

    private Context mContext;
    private BluetoothDevice mDevice;
    private BluetoothGattCallback mInnerGattCallback;
    private BluetoothGattCallback mUserGattCallback;
    private BleCallback mUserBleCallback;
    private BusinessCallback businessCallback;
    private ServiceUuid target_service;

    private BluetoothGatt mGatt;
    private BluetoothGattCharacteristic mWriteChar;
    private final Lock mWriteLock;
    //or read char 未改名
    private BluetoothGattCharacteristic mNotifyChar;

    private int mBleMTU = -1;

    private boolean mEncrypted = false;
    private boolean mChecksum = false;

    private boolean mRequireAck = false;

    private ExecutorService mThreadPool;
    private Handler mUIHandler;

    private int mConnectState = BluetoothGatt.STATE_DISCONNECTED;

    private volatile short fragCtrl = 0;

    private volatile short seq = 1;
    //已经接收read回调
    private volatile boolean hasReceiveSeq = false;

    private volatile byte receiveSeq = 0;

    private int readOutTime = 0;
    private int readErrorTime = 1;
    //是否读取出错
    private boolean readError = false;
    //单次分包接受完成 就是一条完整wifi信息
    private volatile boolean dateGetALl = false;
    //是否需要进行对分包的下一片数据进行接受
    private volatile boolean nextDate2Get = false;

    private volatile String allResult = "";
    //数据全部接收
    private volatile boolean stopGetFate = false;
    //数据类型标志位
    private volatile String resultTypeKey = "";

    private int mType;

    private volatile int data_size = 0;
    private volatile int read_data_size = 0;

    private BleWriteCallback mbleWriteCallback;
    private BleReadCallback mbleReadCallback;
    private BleNotifyCallback mbleNotifyCallback;
    private BleIndicateCallback mbleIndicateCallback;

    Timer timer = new Timer();
    TimerTask task = new TimerTask()        //创建定时器任务对象，必须实现run方法，在该方法中定义用户任务
    {
        @Override
        public void run()
        {
            if (mUserBleCallback != null) {
                mUserBleCallback.onThroughput(data_size,read_data_size);
            }
            setDateSize(-data_size);
            setReadDateSize(-read_data_size);
        }
    };

    private synchronized void setDateSize(int num){
        data_size += num;
    }

    private synchronized void setReadDateSize(int num){
        read_data_size += num;
    }

    BleClientImpl(BleClient client, Context context, BluetoothDevice device , String uuidPreference, ServiceUuid serviceUuid) {
        mClient = client;
        mContext = context;
        mDevice = device;
        mInnerGattCallback = new InnerGattCallback();

        //改用ThreadExecutor创建可以比秒oom 但Executors压测未发现oom
        mThreadPool = Executors.newSingleThreadExecutor();
        mUIHandler = new Handler(Looper.getMainLooper());

        mWriteLock = new ReentrantLock(true);
        target_service = serviceUuid;
    }

    void setGattCallback(BluetoothGattCallback callback) {
        mUserGattCallback = callback;
    }

    void setBleCallback(BleCallback callback) {
        mUserBleCallback = callback;
        timer.schedule(task,1000,1000);                //启动定时器
    }

    void setBusinessCallback(BusinessCallback callback) {
        businessCallback = callback;
    }

    synchronized void connect() {
        if (mThreadPool == null) {
            throw new IllegalStateException("The BleClient has closed");
        }
        if (mGatt != null) {
            mGatt.close();
            mGatt = null;
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            // 双模模式
            mGatt = mDevice.connectGatt(mContext, false, mInnerGattCallback, BluetoothDevice.TRANSPORT_LE);
        } else {
            mGatt = mDevice.connectGatt(mContext, false, mInnerGattCallback);
        }
    }

    synchronized void close() {
        mConnectState = BluetoothGatt.STATE_DISCONNECTED;
        synchronized (mWriteLock) {
            mWriteLock.notifyAll();
        }
        if (mThreadPool != null) {
            mThreadPool.shutdownNow();
            mThreadPool = null;
        }
        if (mGatt != null) {
            mGatt.close();
            mGatt = null;
        }
        mNotifyChar = null;
        mWriteChar = null;

        mClient = null;
        mUserBleCallback = null;
        mInnerGattCallback = null;
        mUserGattCallback = null;
        mContext = null;
        mDevice = null;
        timer.cancel();
    }

    void requestDeviceVersion() {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                __requestDeviceVersion();
            }
        });
    }

    void requestDeviceStatus() {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                __requestDeviceStatus();
            }
        });
    }

    void negotiateSecurity() {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                __negotiateSecurity();
            }
        });
    }

    void configure(final BlufiConfigureParams params) {
        stopGetFate = true;
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                __configure(params);
            }
        });
    }

    void requestDeviceWifiScan() {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                __requestDeviceWifiScan();
            }
        });
    }

    void postCustomData(final byte[] data) {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                __postCustomData(data);
            }
        });
    }

    void requestCloseConnection() {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                __requestCloseConnection();
            }
        });
    }

    private boolean isConnected() {
        return mConnectState == BluetoothGatt.STATE_CONNECTED;
    }

    private void gattWrite(byte[] data) throws InterruptedException {
        readErrorTime = 1;
        mWriteLock.lock();
        Runnable TimeoutRunnable = () -> {
            //设置超时１秒钟　写入超过一秒钟自动释放锁　进行读操作
            new Thread(() -> {
                synchronized (mWriteLock) {
                    mWriteLock.notify();
                }
            }).start();
        };

        try {
            if (!isConnected()) {
                return;
            }
            synchronized (mWriteLock) {
                mWriteChar.setValue(data);
                setDateSize(data.length);
                //add write fail callback
                mGatt.writeCharacteristic(mWriteChar);
                mUIHandler.postDelayed(TimeoutRunnable,1234);
                mWriteLock.wait();

            }
        } finally {
            mUIHandler.removeCallbacks(TimeoutRunnable);
            // 关闭锁
            mWriteLock.unlock();
        }
    }

    public void setMtu(int mtu) {
        mBleMTU = mtu - 4;
    }

    //对于返回值并未做处理
    private boolean post(boolean encrypt, boolean checksum, boolean requireAck, int type, byte[] data) throws InterruptedException {

        fragCtrl=0;
        mType = type;

        if (data == null || data.length == 0) {
            //全局变量seq
            return PostNonData(encrypt, checksum, requireAck, type);
        } else {
            //上面的是传输指令 下面是数据
            return postContainData(encrypt, checksum, requireAck, type, data);
        }
    }

    private boolean PostNonData(boolean encrypt, boolean checksum, boolean requireAck, int type) throws InterruptedException {
        byte[] postBytes = treatyCommandBytes(type,"1234".getBytes());
        Log.w(TAG, "PostNonData:"+System.currentTimeMillis());
        writeAndRead(postBytes);
        return !requireAck ;
    }

    private boolean postContainData(boolean encrypt, boolean checksum, boolean requireAck, int type, byte[] data) throws InterruptedException {
        if (data==null){
            return false;
        }
        Log.d("date length", String.valueOf(data.length));
        int length = new Integer(data.length);
        ByteArrayInputStream dataIS = new ByteArrayInputStream(data);
        ByteArrayOutputStream postOS = new ByteArrayOutputStream();
        byte[] dateBuf = new byte[0];
        if (fragCtrl==0){
            dateBuf  = new byte[mBleMTU];
        }
        while (true) {

            if (fragCtrl==0){
                dateBuf  = new byte[mBleMTU -19];
            }else {
                dateBuf  = new byte[mBleMTU -9];
            }

            int read = dataIS.read(dateBuf, 0, dateBuf.length);
            if (read == -1) {
                break;
            }
            //what is this 多余
            postOS.write(dateBuf, 0, read);
            boolean frag = dataIS.available() > 0;
            byte[] tempData = postOS.toByteArray();
            postOS.reset();
            postOS.write(tempData, 0, tempData.length);
            byte[] postBytes = treatyPostDataBytes(frag,type,postOS.toByteArray(),data.length);
            postOS.reset();

            writeAndRead(postBytes);
            length = length - dateBuf.length;

            if (frag) {
                if (requireAck) {
                    return false;
                }
                sleep(10L);
            } else {
                return !requireAck ;
            }
        }
        fragCtrl=0;
        return true;
    }

    private byte[] treatyCommandBytes(int type,byte[] data) {
        ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
        byte dateb = 0;
        //retry frag prot ack type version
        dateb = bitClear(dateb,7);
        dateb = bitClear(dateb,6);
        dateb = bitClear(dateb,5);
        dateb = bitClear(dateb,4);
        dateb = bitClear(dateb,2);
        dateb = bitClear(dateb,3);
        dateb = bitClear(dateb,1);
        dateb = bitClear(dateb,0);

        if (seq>127) {
            seq=0;
        }
        byteOS.write(dateb);//ctrl
        byteOS.write(seq);//SEQ

        int frag_ctrl =0;
        frag_ctrl = inBitSet(frag_ctrl,15);

        byteOS.write((byte)(frag_ctrl & 0xff));//不分包 直接结束
        byteOS.write((byte)(frag_ctrl >> 8));//Frag CTRL

        byteOS.write(4&0xff);
        byteOS.write(4>>0x08);//total length
        byteOS.write(14);

        byteOS.write(1);//enc ctrl
        byteOS.write(1);//ctr
        byteOS.write(1);//ctr
        byteOS.write(1);//ctr
        byteOS.write(1);//ctr

        byteOS.write(2);//mac
        byteOS.write(2);//mac
        byteOS.write(2);//mac
        byteOS.write(2);//mac

        byteOS.write(type);//cmd id
        byteOS.write(data, 0, data.length);

        return byteOS.toByteArray();
    }

    private byte[] treatyPostDataBytes(boolean frag, int type, byte[] data, int length) {
        ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
        byte dateb = 0;

        dateb = bitClear(dateb,0);
        if (length<= mBleMTU){
            dateb = bitClear(dateb,1);
        }else {
            dateb = bitSet(dateb,1);
        }
        dateb = bitSet(dateb,2);
        dateb = bitClear(dateb,3);
        dateb = bitClear(dateb,5);
        dateb = bitSet(dateb,4);

        dateb = bitClear(dateb,6);
        dateb = bitSet(dateb,7);

        byteOS.write(dateb);//ctrl
        if (seq>127){ seq=0; }
        byteOS.write(seq);//SEQ

        if (fragCtrl==0){

            byteOS.write(fragCtrl&0xff);
            byteOS.write(fragCtrl>>0x08);//Frag CTRL
            byteOS.write(length&0xff);
            byteOS.write(length>>0x08);//total length

            byteOS.write(data.length+9+2);

            byteOS.write(1);//enc ctrl


            byteOS.write(1);//ctr
            byteOS.write(1);//ctr
            byteOS.write(1);//ctr
            byteOS.write(1);//ctr

            byteOS.write(2);//mac
            byteOS.write(2);//mac
            byteOS.write(2);//mac
            byteOS.write(2);//mac
        }else {
            if (frag){
                //没结束
                byteOS.write(fragCtrl&0xff);
                byteOS.write(fragCtrl>>0x08);//Frag CTRL
            }else {
                int dater = inBitSet(fragCtrl,15);
                byteOS.write(fragCtrl&0xff);
                byteOS.write(dater>>0x08);//Frag CTRL
            }

            byteOS.write(data.length+3);
            byteOS.write(1);//enc ctrl
            //byteOS.write(1);//enc ctrl
            //byteOS.write(1);//ctr
            //byteOS.write(1);//mac
        }
        //byteOS.write(data.length&0xff);
        //byteOS.write(data.length>>0x08);
        byteOS.write(data.length);//data length
        byteOS.write(type);//data type
        if (data != null) {
            byteOS.write(data, 0, data.length);
        }

        return byteOS.toByteArray();
    }

    private synchronized void writeAndRead(byte[] postBytes) throws InterruptedException {
        if (mConnectState == BluetoothGatt.STATE_DISCONNECTED)
        {
            Looper.prepare();
            Toast.makeText(mContext,"BluetoothGatt STATE DISCONNECTED !", Toast.LENGTH_LONG).show();
            Looper.loop();
            return;
        }
        gattWrite(postBytes);
        //根据ｍｔｕ大小来决定等待时间
        //sleep(10);
        hasReceiveSeq = false;
        //是否需要接受回调
        int code = bitGet(postBytes[0],3);

        if (code==0){
            int time = 0;
            readError = false;
            boolean readSuc = mGatt.readCharacteristic(mNotifyChar);
            Log.w(TAG,"readSuc:"+readSuc);
            //等待回调关闭锁
            if (!readSuc){
                mUserBleCallback.onError(mClient,BleCallback.CODE_WRITE_DATA_FAILED);
                return;
            }
            while (!hasReceiveSeq||seq!=receiveSeq){
                sleep(1);
                //yield()
                //暂停当前执行的线程对象，并执行其他线程。这个暂停会放弃cpu资源，放弃的时间不确定。
                if (readError){
                    Log.d(TAG,"readError break");
                    break;
                }

                if (time==2500){
                    //无回答
                    Log.d("readErrorTime", "无回答"+readOutTime);
                    byte b = bitSet(postBytes[0],7);
                    postBytes[0] = b;
                    readOutTime++;
                    Log.d("readErrorTime", "status==4");
                    if (readOutTime<10){
                        sleep(30*readOutTime);
                        writeAndRead(postBytes);
                    }else {
                        //回调给 用主线程
                        readOutTime = 1;
                        mUserBleCallback.onError(mClient,BleCallback.CODE_WRITE_DATA_FAILED);
                    }

                    return;
                }
                time++;
            }
            if (readError){
                //改变重发标志位
                byte b = bitSet(postBytes[0],7);
                postBytes[0] = b;

                writeAndRead(postBytes);
                return;
            }
            if (seq==receiveSeq){
                seq++;
                fragCtrl++;
            }else {
                byte b = bitSet(postBytes[0],7);
                postBytes[0] = b;

                writeAndRead(postBytes);
            }
            // 读取下一个frag
            if (nextDate2Get){
                nextDate2Get = false;
                try {
                    writeAndRead(treatyCommandBytes(mType,"1234".getBytes()));
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    // 使用DH 传输密钥(已改为蓝牙绑定)
    private void __negotiateSecurity() {
        if (mGatt == null) {
            return;
        }
        if (mGatt.getDevice().getBondState() == BOND_BONDED){
            onConfigureResult(BleCallback.CODE_NEG_ERR_SECURITY);
            return;
        }
        if (mGatt.connect()){
            mGatt.getDevice().createBond();
        }

    }

    private void __configure(BlufiConfigureParams params) {
        int opMode = params.getOpMode();
        switch (opMode) {
            case OP_MODE_STA:
                if (!postStaWifiInfo(params)) {
                    onConfigureResult(BleCallback.CODE_CONF_ERR_POST_STA);
                    return;
                }

                onConfigureResult(BleCallback.STATUS_SUCCESS);
                return;
        }
    }

    private void onConfigureResult(final int status) {
        mUIHandler.post(() -> {
            if (mUserBleCallback != null) {
                mUserBleCallback.onConfigureResult(mClient, status);
            }
        });
    }

    private boolean postStaWifiInfo(BlufiConfigureParams params) {
        try {
            int ssidType = Type.Data.SUBTYPE_STA_WIFI_SSID;
            byte[] ssidBytes = params.getStaSSIDBytes();
            if (!post(mEncrypted, mChecksum, mRequireAck, ssidType, ssidBytes)) {
                return false;
            }
            sleep(10);

            int pwdType = Type.Data.SUBTYPE_STA_WIFI_PASSWORD;
            if (!post(mEncrypted, mChecksum, mRequireAck, pwdType, params.getStaPassword().getBytes())) {
                return false;
            }
            sleep(10);

            int comfirmType = Type.Ctrl.SUBTYPE_CONNECT_WIFI;

            return post(false, false, mRequireAck, comfirmType, (byte[]) null);
        } catch (InterruptedException e) {
            Log.w(TAG, "postStaWifiInfo: interrupted");
            Thread.currentThread().interrupt();
            return false;
        }
    }

    private void __requestDeviceVersion() {
        int type =  Type.Ctrl.SUBTYPE_GET_VERSION;
        boolean request;
        try {
            resultTypeKey = "DeviceVersion";
            request = post(mEncrypted, mChecksum, false, type, (byte[]) null);
        } catch (InterruptedException e) {
            Log.w(TAG, "post __requestDeviceVersion interrupted");
            Thread.currentThread().interrupt();
        }
    }

    private void onVersionResponse(final int status, final BleVersionResponse response) {
        mUIHandler.post(() -> {
            if (mUserBleCallback != null) {
                mUserBleCallback.onDeviceVersionResponse(mClient, status, response);
            }
        });
    }

    private void __requestDeviceStatus() {
        int type =  Type.Ctrl.SUBTYPE_GET_WIFI_STATUS;
        boolean request;
        try {
            resultTypeKey = "wifiState";
            request = post(mEncrypted, mChecksum, false, type, (byte[]) null);
        } catch (InterruptedException e) {
            Log.w(TAG, "post requestDeviceStatus interrupted");
            request = false;
            Thread.currentThread().interrupt();
        }

        if (!request) {
            onStatusResponse(BleCallback.CODE_WRITE_DATA_FAILED, null);
        }
    }

    private void onStatusResponse(final int status, final String wifiState) {
        mUIHandler.post(() -> {
            if (mUserBleCallback != null) {
                mUserBleCallback.onDeviceStatusResponse(mClient, status, wifiState);
            }
        });
    }

    private void __requestDeviceWifiScan() {
        try {
            allResult="";
            post(mEncrypted, mChecksum, mRequireAck, Type.Ctrl.SUBTYPE_GET_WIFI_LIST, null);
            sleep(2500);
            stopGetFate = false;
            //单线程会卡死此线程
            new Thread(() -> {
                while (!stopGetFate){

                    dateGetALl=false;
                    mThreadPool.submit(new ThrowableRunnable() {
                        @Override
                        void execute() throws InterruptedException {
                            resultTypeKey = "WifiScan";
                            post(mEncrypted, mChecksum, mRequireAck, Type.Ctrl.SUBTYPE_GET_WIFI_LIST_DATA, null);
                        }
                    });

                    while (!dateGetALl){
                        //传输密码或者其他数据时　改变stopGetFate 在configure中改变
                        sleep(1);
                    }
                }
            }).start();

        } catch (InterruptedException e) {
            Log.w(TAG, "requestDeviceWifiScan interrupted");
            Thread.currentThread().interrupt();
        }
    }

    private void onDeviceScanResult(final int status, final String results) {
        mUIHandler.post(() -> {
            if (mUserBleCallback != null) {
                mUserBleCallback.onDeviceScanResult(mClient, status, results);
            }
        });
    }

    private void __postCustomData(byte[] data) {
        int type = Type.Data.SUBTYPE_CUSTOM_DATA;
        try {
            boolean suc = post(mEncrypted, mChecksum, mRequireAck, type, data);
        } catch (InterruptedException e) {
            Log.w(TAG, "post postCustomData interrupted");
            Thread.currentThread().interrupt();
        }
    }

    private void onPostCustomDataResult(final int status, final String data) {
        mUIHandler.post(() -> {
            if (mUserBleCallback != null) {
                mUserBleCallback.onPostCustomDataResult(mClient, status, data);
            }
        });
    }

    private void __requestCloseConnection() {
        int type = Type.Ctrl.SUBTYPE_DISCONNECT_WIFI;
        try {
            post(mEncrypted, mChecksum, false, type, (byte[])null);
        } catch (InterruptedException e) {
            Log.w(TAG, "post requestCloseConnection interrupted");
            Thread.currentThread().interrupt();
        }
    }

    public BluetoothGatt getGatt() {
        return mGatt;
    }

    public int getMtu() {
        return mBleMTU;
    }

    public static byte[] objectToByte(Object obj) throws Exception {
        ObjectOutputStream oos = null;
        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            oos = new ObjectOutputStream(bos);
            oos.writeObject(obj);
            return bos.toByteArray();
        } finally {
            if (oos != null) oos.close();
        }
    }


    public void postOtaBinInfo(byte[] data,int type) {
//        Log.e(TAG, Arrays.toString(objectToByte(otaInfo)));
//        Log.e(TAG, String.valueOf((objectToByte(otaInfo).length)));
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() throws Exception {
                try {
                    post(mEncrypted, mChecksum, mRequireAck, type, data);
                } catch (InterruptedException e) {
                    Log.w(TAG, "post postCustomData interrupted");
                    Thread.currentThread().interrupt();
                }
            }
        });
    }

    public void postOtaBinData(byte[] data) {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                int type = Type.Data.SUBTYPE_OTA_DATA;
                try {
                    post(mEncrypted, mChecksum, mRequireAck, type, data);
                } catch (InterruptedException e) {
                    Log.w(TAG, "post postOtaBinData interrupted");
                    Thread.currentThread().interrupt();
                }
            }
        });
    }

    public void send_ble_CMD_ZB_RESET() {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                int type = Type.Ctrl.SUBTYPE_CMD_ZB_RESET;
                try {
                    post(mEncrypted, mChecksum, mRequireAck, type, null);
                } catch (InterruptedException e) {
                    Log.w(TAG, "post postCustomData interrupted");
                    Thread.currentThread().interrupt();
                }
            }
        });
    }

    public void send_ble_DATA_ZB_LINKKEY() {
        mThreadPool.submit(new ThrowableRunnable() {
            @Override
            void execute() {
                int type = Type.Data.SUBTYPE_DATA_ZB_LINKKEY;
                try {
                    post(mEncrypted, mChecksum, mRequireAck, type, "0123456789123456".getBytes());
                } catch (InterruptedException e) {
                    Log.w(TAG, "post postCustomData interrupted");
                    Thread.currentThread().interrupt();
                }
            }
        });
    }


    private abstract static class ThrowableRunnable implements Runnable {
        @Override
        public void run() {
            try {
                execute();
            } catch (Exception e) {
                e.printStackTrace();
                onError(e);
            }
        }

        abstract void execute() throws Exception;

        void onError(Exception e) {
        }
    }

    private void sleep(long timeout) {
        try {
            Thread.sleep(timeout);
        } catch (InterruptedException e) {
            Log.w(TAG, "sleep thread.currentThread().interrupt()");
            Thread.currentThread().interrupt();
        }
    }

    public byte[] subBytes(byte[] src, int begin, int count) {
        byte[] bs = new byte[count];
        System.arraycopy(src, begin, bs, 0, begin + count - begin);
        return bs;
    }

    private class InnerGattCallback extends BluetoothGattCallback {

        public void onDescriptorRead(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            if (mUserGattCallback != null) {
                mUserGattCallback.onDescriptorRead(gatt, descriptor, status);
            }
        }

        public void onDescriptorWrite(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            if (mUserGattCallback != null) {
                mUserGattCallback.onDescriptorWrite(gatt, descriptor, status);
            }
        }

        public void onReliableWriteCompleted(BluetoothGatt gatt, int status) {
            if (mUserGattCallback != null) {
                mUserGattCallback.onReliableWriteCompleted(gatt, status);
            }
        }

        public void onReadRemoteRssi(BluetoothGatt gatt, int rssi, int status) {
            if (mUserGattCallback != null) {
                mUserGattCallback.onReadRemoteRssi(gatt, rssi, status);
            }
        }

        @TargetApi(Build.VERSION_CODES.O)
        public void onPhyUpdate(BluetoothGatt gatt, int txPhy, int rxPhy, int status) {
            if (mUserGattCallback != null) {
                mUserGattCallback.onPhyUpdate(gatt, txPhy, rxPhy, status);
            }
        }

        @TargetApi(Build.VERSION_CODES.O)
        public void onPhyRead(BluetoothGatt gatt, int txPhy, int rxPhy, int status) {
            if (mUserGattCallback != null) {
                mUserGattCallback.onPhyRead(gatt, txPhy, rxPhy, status);
            }
        }

        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            mConnectState = newState;
            mBleMTU = -1;
            if (status == BluetoothGatt.GATT_SUCCESS) {
                if (newState == BluetoothProfile.STATE_CONNECTED) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                        //设置高低功耗
                        gatt.requestConnectionPriority(BluetoothGatt.CONNECTION_PRIORITY_HIGH);
                    }
                    gatt.discoverServices();
                }
            }

            if (mUserGattCallback != null) {
                mUserGattCallback.onConnectionStateChange(gatt, status, newState);
            }
            if (businessCallback != null) {
                businessCallback.onConnectionStateChange(status, newState);
            }

        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            BluetoothGattService service = null;
            BluetoothGattCharacteristic writeChar = null;
            BluetoothGattCharacteristic notifyChar = null;

            if (status == BluetoothGatt.GATT_SUCCESS) {
                // 此处修改未协定的UUID
                service = gatt.getService(target_service.getUUID_SERVICE());
                if (service != null) {
                    writeChar = service.getCharacteristic(target_service.getUUID_WRITE_CHARACTERISTIC());
                    notifyChar = service.getCharacteristic(target_service.getUUID_NOTIFICATION_CHARACTERISTIC());
                    if (notifyChar != null) {
                        gatt.setCharacteristicNotification(notifyChar, true);
                    }
                }
                //读数据的服务和characteristic
                //BluetoothGattService readMnotyGattService = gatt.getService(BlufiParameter.UUID_NOTIFICATION_CHARACTERISTIC);
                ///readCharacteristic = service.getCharacteristic(BlufiParameter.UUID_NOTIFICATION_CHARACTERISTIC);

                mWriteChar = writeChar;
                mNotifyChar = notifyChar;
            }

            if (mUserGattCallback != null) {
                mUserGattCallback.onServicesDiscovered(gatt, status);
            }
            if (mUserBleCallback != null) {
                final BluetoothGattService cbService = service;
                final BluetoothGattCharacteristic cbWriteChar = writeChar;
                final BluetoothGattCharacteristic cbNotifyChar = notifyChar;
                mUIHandler.post(() -> {
                    if (mUserBleCallback != null) {
                        mUserBleCallback.onGattPrepared(mClient, gatt, cbService, cbWriteChar, cbNotifyChar);
                    }
                });
            }
        }

        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
//            Log.e(TAG,"onCharacteristicChanged CALLBACK");
            setReadDateSize(characteristic.getValue().length);
            if (mbleNotifyCallback!=null){
//                Log.d(TAG,characteristic.getUuid().toString());
//                Log.d(TAG,mbleNotifyCallback.getKey());
//                Log.w(TAG, String.valueOf(characteristic.getValue().length));
                if (characteristic.getUuid().toString().equalsIgnoreCase(mbleNotifyCallback.getKey())) {
                    mbleNotifyCallback.onCharacteristicChanged(characteristic.getValue());
                }
            }

            if (mbleIndicateCallback != null) {
                if (characteristic.getUuid().toString().equalsIgnoreCase(mbleIndicateCallback.getKey())) {
                    mbleIndicateCallback.onCharacteristicChanged(characteristic.getValue());
                }
            }

            if (!characteristic.equals(mNotifyChar)) {
                return;
            }

//            Log.d("Changed is ", Arrays.toString(characteristic.getValue()));
        }

        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            if (!characteristic.equals(mWriteChar)) {
                return;
            }
            //Write back interface
            synchronized (mWriteLock) {
                mWriteLock.notify();
            }

            if (mUserGattCallback != null) {
                mUserGattCallback.onCharacteristicWrite(gatt, characteristic, status);
            }

            if (mbleWriteCallback != null) {
                if (status == BluetoothGatt.GATT_SUCCESS) {
                    mbleWriteCallback.onWriteSuccess(1, 1, characteristic.getValue());
                } else {
                    mbleWriteCallback.onWriteFailure(new GattException(status));
                }
            }

        }

        @TargetApi(Build.VERSION_CODES.LOLLIPOP)
        public void onMtuChanged(BluetoothGatt gatt, int mtu, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                mBleMTU = mtu - 4; // Three bytes BLE header, one byte reserved
            }
            if (mUserGattCallback != null) {
                mUserGattCallback.onMtuChanged(gatt, mtu, status);
            }
        }

        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {

            if (mbleReadCallback != null) {
                if (status == BluetoothGatt.GATT_SUCCESS) {
                    mbleReadCallback.onReadSuccess(characteristic.getValue());
                    return;
                } else {
                    mbleReadCallback.onReadFailure(new GattException(status));
                }
            }

            if (status==4){
                //设备状态繁忙
                //增加回调　繁忙超过若干次　重新写处理
                readErrorTime++;
                if (readErrorTime<10){
                    sleep(30*readErrorTime);
                    mGatt.readCharacteristic(mNotifyChar);
                }else {
                    //回调给 用主线程
                    readErrorTime = 1;
                    mUserBleCallback.onError(mClient,BleCallback.CODE_WRITE_DATA_FAILED);

                }
                return;

            }else if (status!=0){
                readError = true;
                return;
            }

            byte[] data = characteristic.getValue();
            if (data==null||data.length==0){
                readError = true;
            }else {
                receiveSeq = data[1];
//                Log.d(TAG, "receiveSeq_is"+String.valueOf(receiveSeq)+"_Seq_is"+String.valueOf(seq));

                if (seq!=receiveSeq){
                    readError = true;
                    return;
                }
                int frag_ctrl = ((int)data[3] << 8) | data[2];//低位:第几次
                frag_ctrl &= 0x7fff;
//                Log.d("frag_ctrl", String.valueOf(frag_ctrl));

                byte ackCode;
                if (frag_ctrl==0){
                    ackCode = data[17];
                    int dateLength = byteToInteger(data[6]);
                    if (dateLength<11){
//                        Log.d("dateLength", "get error bec dateLength");
                        //dateGetALl = true;
                        readError = true;
                        return;
                    }
                }else {
                    ackCode = data[7];
                    int dateLength = byteToInteger(data[6]);
                    if (dateLength<1){
//                        Log.d("dateLength", "get error bec dateLength");
                        //dateGetALl = true;
                        readError = true;
                        return;
                    }
                }

//                Log.d("->resultTypeKey:::", String.valueOf(resultTypeKey));

                final String copyResultTypeKey = resultTypeKey;

                //是否正确收到
                if (ackCode==SUBTYPE_ACK_SUCCESS){
                }else if (ackCode==SUBTYPE_NOT_READY){
                    sleep(10);
                    readError=true;
                    return;
                }else if (ackCode==SUBTYPE_ERROE_TYPE){
                    hasReceiveSeq = true;
                    stopGetFate = true;
                    dateGetALl = true;
                    return;
                }else  {
                    hasReceiveSeq = false;
                    //解析错误 重新写
                    return;
                }
//                Log.d(TAG, String.valueOf(System.identityHashCode(copyResultTypeKey)));
//
//                Log.d("->copyResultTypeKey:::", String.valueOf(copyResultTypeKey));
//                Log.d(TAG, String.valueOf(System.identityHashCode(resultTypeKey)));

                //开始解析数据　在回调线程中执行　故对加锁
                //无需加锁　copy一下即可
                String dateString;
                if (frag_ctrl==0){
                     dateString = new String(subBytes(data, 18,byteToInteger(data[16])));
                }else {
                     dateString = new String(subBytes(data,8,byteToInteger(data[6])));
                }
//                Log.d("接受到　dateString:", dateString);

                if (dateString.length()==0){
                    hasReceiveSeq = true;
                    resultTypeKey = "";
                    stopGetFate = true;
                    dateGetALl = true;
                    //指令发送成功　回调　使用future ing
                }else {
                    allResult+=dateString;
//                    Log.w("allResult", allResult);

                    int frag_ctrl_high = ((int)data[3] << 8) | data[2];
                    int code = bitGet(frag_ctrl_high,15);//高位
                    if (code==1){
                        hasReceiveSeq = true;
                        resultTypeKey = "";
                        //判断frag_ctrl是否第一次发送，不是的话执行，是的话再判断长度是否为0
                        dateGetALl = true;
//                      Log.d(TAG, String.valueOf(System.identityHashCode(copyResultTypeKey)));
//                      Log.d("->copyResultTypeKey:::", String.valueOf(copyResultTypeKey));
//                      Log.d(TAG, String.valueOf(System.identityHashCode(copyResultTypeKey)));
//                      "next -> resolve date "+"rTypeKey"+
//                      Log.d(TAG, String.valueOf(copyResultTypeKey));
                        try {
//                          Log.d(TAG, String.valueOf("DeviceVersion".equals(String.valueOf(copyResultTypeKey))));
                            if (copyResultTypeKey.equals("wifiState")){
                                onStatusResponse(10010, allResult);
                            }else if ("DeviceVersion".equals(String.valueOf(copyResultTypeKey))){
                                BleVersionResponse response = new BleVersionResponse();
                                response.setVersionValues(allResult);
                                onVersionResponse(BleCallback.STATUS_SUCCESS, response);
                            }else if (copyResultTypeKey.equals("WifiScan")){
                                onDeviceScanResult(10010, allResult);
                            }else {
                                onPostCustomDataResult(10010,allResult);
                            }
                        }catch (Exception e){
//                            Log.e(TAG,"json解析出错 "+e.toString());
                        }finally {
                            allResult="";
                        }
                        if (frag_ctrl==0){

                        }else {

                        }
                    }else {
                        hasReceiveSeq = true;
                        nextDate2Get = true;

                        //下面代码在读等待循环后执行
//                        mThreadPool.submit(new ThrowableRunnable() {
//                            @Override
//                            void execute() {
////                                resultTypeKey = copyResultTypeKey;
//                                byte[] postBytes = treatyCommandBytes(mType,"1234".getBytes());
//                                try {
////                                    Log.w(TAG, "code!=1:"+System.currentTimeMillis());
//                                    writeAndRead(postBytes);
//                                } catch (InterruptedException e) {
//                                    e.printStackTrace();
//                                }
//                            }
//                        });
                    }
                }
            }
        }
    }


    /*------------------------------- main operation ----------------------------------- */

    @Override
    public void stopTask() {
        stopGetFate = true;
        dateGetALl = true;

        //goto stop wifi_list getting
        //shutdown之后不能提交新任务，之前提交的任务会继续执行，而且不能重启启动了，如果还想使用就只能再起一个了。
//        mThreadPool.shutdownNow();
    }

    private UUID formUUID(String uuid) {
        return uuid == null ? null : UUID.fromString(uuid);
    }

    /**
     * read
     */
    @Override
    public void readCharacteristic(String uuid_service, String uuid_read, BleReadCallback callback) {
        BluetoothGattCharacteristic mCharacteristic = null;
        if (uuid_service != null && getGatt() != null) {
            BluetoothGattService mGattService = getGatt().getService(formUUID(uuid_service));
            if (mGattService != null && uuid_read != null) {
                mCharacteristic  = mGattService.getCharacteristic(formUUID(uuid_read));
            }
        }
        if (mCharacteristic != null
                && (mCharacteristic.getProperties() & BluetoothGattCharacteristic.PROPERTY_READ) > 0) {
            mbleReadCallback = callback;
            if (!getGatt().readCharacteristic(mCharacteristic)) {
                if (callback != null)
                    callback.onReadFailure(new OtherException("gatt readCharacteristic fail"));
            }
        } else {
            if (callback != null)
                callback.onReadFailure(new OtherException("this characteristic not support read!"));
        }
    }

    /**
     * write
     */
    @Override
    public void writeCharacteristic(String uuid_service, String uuid_write, byte[] data, BleWriteCallback bleWriteCallback) {
        if (data == null || data.length <= 0) {
            if (bleWriteCallback != null)
                bleWriteCallback.onWriteFailure(new OtherException("the data to be written is empty"));
            return;
        }
        BluetoothGattCharacteristic mCharacteristic = null;
        if (uuid_service != null && getGatt() != null) {
            BluetoothGattService mGattService = getGatt().getService(formUUID(uuid_service));
            if (mGattService != null && uuid_write != null) {
                mCharacteristic  = mGattService.getCharacteristic(formUUID(uuid_write));
            }
        }

        if (mCharacteristic == null
                || (mCharacteristic.getProperties() & (BluetoothGattCharacteristic.PROPERTY_WRITE | BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE)) == 0) {
            if (bleWriteCallback != null)
                bleWriteCallback.onWriteFailure(new OtherException("this characteristic not support write!"));
            return;
        }

        if (mCharacteristic.setValue(data)) {
//            handleCharacteristicWriteCallback(bleWriteCallback, uuid_write);
            readErrorTime = 1;
            if (!mGatt.writeCharacteristic(mCharacteristic)) {
//                writeMsgInit();
                if (bleWriteCallback != null)
                    bleWriteCallback.onWriteFailure(new OtherException("gatt writeCharacteristic fail"));
            }else {
                setDateSize(data.length);
                mbleWriteCallback = bleWriteCallback;
            }
        } else {
            if (bleWriteCallback != null)
                bleWriteCallback.onWriteFailure(new OtherException("Updates the locally stored value of this characteristic fail"));
        }
    }

    @Override
    public void destroyDetailCallback() {
        mbleWriteCallback = null;
        mbleReadCallback = null;
        mbleNotifyCallback = null;
        mbleIndicateCallback = null;
    }

    @Override
    public void enableCharacteristicNotify(BleNotifyCallback callback, String uuid_service, String uuid_notify, boolean isNotify) {
        BluetoothGattCharacteristic mCharacteristic = null;
        if (uuid_service != null && getGatt() != null) {
            BluetoothGattService mGattService = getGatt().getService(formUUID(uuid_service));
            if (mGattService != null && uuid_notify != null) {
                mCharacteristic  = mGattService.getCharacteristic(formUUID(uuid_notify));
            }
        }
        if (mCharacteristic != null
                && (mCharacteristic.getProperties() | BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {

            setCharacteristicNotification(getGatt(), mCharacteristic, isNotify,  callback);
        } else {
            if (callback != null)
                callback.onNotifyFailure(new OtherException("this characteristic not support notify!"));
        }
        mbleNotifyCallback = callback;

    }

    @Override
    public boolean disenableCharacteristicNotify(String uuid_service, String uuid_notify, boolean isNotify) {
        BluetoothGattCharacteristic mCharacteristic = null;
        if (uuid_service != null && getGatt() != null) {
            BluetoothGattService mGattService = getGatt().getService(formUUID(uuid_service));
            if (mGattService != null && uuid_notify != null) {
                mCharacteristic  = mGattService.getCharacteristic(formUUID(uuid_notify));
            }
        }
        mbleNotifyCallback = null;
        if (mCharacteristic != null
                && (mCharacteristic.getProperties() | BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {
            return setCharacteristicNotification(getGatt(), mCharacteristic, false, null);
        } else {
            return false;
        }
    }

    @Override
    public void enableCharacteristicIndicate(BleIndicateCallback callback, String uuid_service, String uuid_indicate) {
        BluetoothGattCharacteristic mCharacteristic = null;
        if (uuid_service != null && getGatt() != null) {
            BluetoothGattService mGattService = getGatt().getService(formUUID(uuid_service));
            if (mGattService != null && uuid_indicate != null) {
                mCharacteristic  = mGattService.getCharacteristic(formUUID(uuid_indicate));
            }
        }
        if (mCharacteristic != null
                && (mCharacteristic.getProperties() | BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {
            setCharacteristicIndication(getGatt(), mCharacteristic,
                    false, true, callback);
        } else {
            if (callback != null)
                callback.onIndicateFailure(new OtherException("this characteristic not support indicate!"));
        }
        mbleIndicateCallback = callback;
    }

    @Override
    public boolean disableCharacteristicIndicate(String uuid_service, String uuid_indicate, boolean b) {
        BluetoothGattCharacteristic mCharacteristic = null;
        if (uuid_service != null && getGatt() != null) {
            BluetoothGattService mGattService = getGatt().getService(formUUID(uuid_service));
            if (mGattService != null && uuid_indicate != null) {
                mCharacteristic  = mGattService.getCharacteristic(formUUID(uuid_indicate));
            }
        }
        if (mCharacteristic != null
                && (mCharacteristic.getProperties() | BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {
            return setCharacteristicIndication(getGatt(), mCharacteristic,
                    false, false, null);
        } else {
            return false;
        }
    }

    /**
     * indicate setting
     */
    private boolean setCharacteristicIndication(BluetoothGatt gatt,
                                                BluetoothGattCharacteristic characteristic,
                                                boolean useCharacteristicDescriptor,
                                                boolean enable,
                                                BleIndicateCallback bleIndicateCallback) {
        if (gatt == null || characteristic == null) {
            if (bleIndicateCallback != null)
                bleIndicateCallback.onIndicateFailure(new OtherException("gatt or characteristic equal null"));
            return false;
        }

        boolean success1 = gatt.setCharacteristicNotification(characteristic, enable);
        if (!success1) {
            if (bleIndicateCallback != null)
                bleIndicateCallback.onIndicateFailure(new OtherException("gatt setCharacteristicNotification fail"));
            return false;
        }

        BluetoothGattDescriptor descriptor;
            descriptor = characteristic.getDescriptor(characteristic.getUuid());
        if (descriptor == null) {
            if (bleIndicateCallback != null)
                bleIndicateCallback.onIndicateFailure(new OtherException("descriptor equals null"));
            return false;
        } else {
            descriptor.setValue(enable ? BluetoothGattDescriptor.ENABLE_INDICATION_VALUE :
                    BluetoothGattDescriptor.DISABLE_NOTIFICATION_VALUE);
            boolean success2 = gatt.writeDescriptor(descriptor);
            if (!success2) {
                if (bleIndicateCallback != null)
                    bleIndicateCallback.onIndicateFailure(new OtherException("gatt writeDescriptor fail"));
            }
            return success2;
        }
    }


    /**
     * notify setting
     */
    private boolean setCharacteristicNotification(BluetoothGatt gatt,
                                                  BluetoothGattCharacteristic characteristic,
                                                  boolean enable,
                                                  BleNotifyCallback bleNotifyCallback) {
        if (gatt == null || characteristic == null) {
            if (bleNotifyCallback != null)
                bleNotifyCallback.onNotifyFailure(new OtherException("gatt or characteristic equal null"));
            return false;
        }

        boolean success1 = gatt.setCharacteristicNotification(characteristic, enable);
        if (!success1) {
            if (bleNotifyCallback != null)
                bleNotifyCallback.onNotifyFailure(new OtherException("gatt setCharacteristicNotification fail"));
            return false;
        }

        BluetoothGattDescriptor descriptor = characteristic.getDescriptor(UUID.fromString("00002902-0000-1000-8000-00805f9b34fb"));
        Log.w(TAG, String.valueOf(descriptor==null));
        if (descriptor == null) {
            if (bleNotifyCallback != null)
                bleNotifyCallback.onNotifyFailure(new OtherException("descriptor equals null"));
            return false;
        } else {
            descriptor.setValue(enable ? BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE :
                    BluetoothGattDescriptor.DISABLE_NOTIFICATION_VALUE);
            boolean success2 = gatt.writeDescriptor(descriptor);
            if (!success2) {
                if (bleNotifyCallback != null)
                    bleNotifyCallback.onNotifyFailure(new OtherException("gatt writeDescriptor fail"));
            }
            return success2;
        }
    }

}